@@ -24,7 +24,14 @@
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
- *
+//
+// Aegisub Project http://www.aegisub.org/
+
+/// @file MatroskaParser.c
+/// @brief Haali's low-level Matroska-parsing library
+/// @ingroup video_input
+///
+
  */
 
 #include <stdlib.h>
@@ -560,7 +567,7 @@ static void   readbytes(MatroskaFile *mf,void *buffer,int len) {
 }
 
 static void   skipbytes(MatroskaFile *mf,ulonglong len) {
-  int	    nb = mf->buflen - mf->bufpos;
+  unsigned int	    nb = mf->buflen - mf->bufpos;
 
   if (nb > len)
     nb = (int)len;
@@ -831,7 +838,7 @@ shift:
 }
 
 static void readString(MatroskaFile *mf,ulonglong len,char *buffer,int buflen) {
-  int	  nread;
+  unsigned int	  nread;
 
   if (buflen<1)
     errorjmp(mf,"Invalid buffer size in readString: %d",buflen);
@@ -904,6 +911,7 @@ static void readLangCC(MatroskaFile *mf, ulonglong len, char lcc[4]) {
 #define	STRGETA(f,v,len)  STRGETF(f,v,len,myalloca)
 #define	STRGETM(f,v,len)  STRGETF(f,v,len,f->cache->memalloc)
 
+/* Removed because it's not used.
 static int  IsWritingApp(MatroskaFile *mf,const char *str) {
   const char  *cp = mf->Seg.WritingApp;
   if (!cp)
@@ -913,7 +921,7 @@ static int  IsWritingApp(MatroskaFile *mf,const char *str) {
 
   return !*str;
 }
-
+*/
 static void parseEBML(MatroskaFile *mf,ulonglong toplen) {
   ulonglong v;
   char	    buf[32];
@@ -3310,3 +3318,4 @@ const char	  *cs_GetLastError(CompressedStream *cs)
   return cs->errmsg;
 }
 #endif
+
