@@ -25,172 +25,255 @@
 // ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 // POSSIBILITY OF SUCH DAMAGE.
 //
-// -----------------------------------------------------------------------------
-//
-// AEGISUB
-//
-// Website: http://www.aegisub.net/
-// Contact: mailto:jiifurusu@gmail.com
-//
+// Aegisub Project http://www.aegisub.org/
 
+/// @file audio_player_dsound2.cpp
+/// @brief New DirectSound-based audio output
+/// @ingroup audio_output
+///
 
-///////////
-// Headers
 #include "config.h"
 
 #ifdef WITH_DIRECTSOUND
-
-#include <wx/wxprec.h>
-#include <mmsystem.h>
-#include <dsound.h>
-#include <process.h>
-#include "include/aegisub/audio_provider.h"
-#include "utils.h"
-#include "main.h"
-#include "frame_main.h"
-#include "options.h"
 #include "audio_player_dsound2.h"
 
+#include "audio_controller.h"
+#include "include/aegisub/audio_provider.h"
+#include "frame_main.h"
+#include "main.h"
+#include "options.h"
+#include "utils.h"
 
+#include <libaegisub/scoped_ptr.h>
+#include <libaegisub/log.h>
+#include <libaegisub/util.h>
+
+#include <mmsystem.h>
+#include <process.h>
+#include <dsound.h>
+
+/// @brief RAII support class to init and de-init the COM library
 struct COMInitialization {
+
+	/// Flag set if an inited COM library is managed
 	bool inited;
 
+	/// @brief Constructor, sets inited false
 	COMInitialization()
 	{
 		inited = false;
 	}
 
+	/// @brief Destructor, de-inits COM if it is inited
 	~COMInitialization()
 	{
 		if (inited) CoUninitialize();
 	}
 
+	/// @brief Initialise the COM library as single-threaded apartment if isn't already inited by us
 	void Init()
 	{
 		if (!inited)
 		{
-			if (FAILED(CoInitialize(NULL)))
+			if (FAILED(CoInitialize(nullptr)))
 				throw std::exception();
 			inited = true;
 		}
 	}
 };
 
-
+/// @class COMObjectRetainer
+/// @brief Simple auto_ptr-like class for COM objects
 template<class T>
 struct COMObjectRetainer {
+	/// Managed object
 	T *obj;
 
+	/// @brief Constructor for null object
 	COMObjectRetainer()
 	{
 		obj = 0;
 	}
 
+	/// @brief Constructor to take object immediately
+	/// @param _obj Object to manage
 	COMObjectRetainer(T *_obj)
 	{
 		obj = _obj;
 	}
 
+	/// @brief Destructor, releases object if there is one
 	~COMObjectRetainer()
 	{
 		if (obj) obj->Release();
 	}
 
+	/// @brief Dereference the managed object
+	/// @return The managed object
 	T * operator -> ()
 	{
 		return obj;
 	}
 };
 
-
-struct Win32KernelHandle {
-	/// HANDLE value being managed
-	HANDLE handle;
-
+/// @brief RAII wrapper around Win32 HANDLE type
+struct Win32KernelHandle : public agi::scoped_holder<HANDLE, BOOL (__stdcall *)(HANDLE)> {
 	/// @brief Create with a managed handle
 	/// @param handle Win32 handle to manage
 	Win32KernelHandle(HANDLE handle = 0)
-		: handle(handle)
+	: scoped_holder(handle, CloseHandle)
 	{
 	}
 
-	/// @brief Destructor, closes the managed handle
-	~Win32KernelHandle()
+	Win32KernelHandle& operator=(HANDLE new_handle)
 	{
-		if (handle) CloseHandle(handle);
+		scoped_holder::operator=(new_handle);
+		return *this;
 	}
-
-	/// @brief Returns the managed handle
-	operator HANDLE () const { return handle; }
 };
 
-
-
+/// @class DirectSoundPlayer2Thread
+/// @brief Playback thread class for DirectSoundPlayer2
+///
+/// Not based on wxThread, but uses Win32 threads directly
 class DirectSoundPlayer2Thread {
+	/// @brief Win32 thread entry point
+	/// @param parameter Pointer to our thread object
+	/// @return Thread return value, always 0 here
 	static unsigned int __stdcall ThreadProc(void *parameter);
+	/// @brief Thread entry point
 	void Run();
 
+	/// @brief Fill audio data into a locked buffer-pair and unlock the buffers
+	/// @param buf1        First buffer in pair
+	/// @param buf1sz      Byte-size of first buffer in pair
+	/// @param buf2        Second buffer in pair, or null
+	/// @param buf2sz      Byte-size of second buffer in pair
+	/// @param input_frame First audio frame to fill into buffers
+	/// @param bfr         DirectSound buffer object owning the buffer pair
+	/// @return Number of bytes written
 	DWORD FillAndUnlockBuffers(void *buf1, DWORD buf1sz, void *buf2, DWORD buf2sz, int64_t &input_frame, IDirectSoundBuffer8 *bfr);
 
+	/// @brief Check for error state and throw exception if one occurred
 	void CheckError();
 
+
+	/// Win32 handle to the thread
 	Win32KernelHandle thread_handle;
 
-	// Used to signal state-changes to thread
-	Win32KernelHandle
-		event_start_playback,
-		event_stop_playback,
-		event_update_end_time,
-		event_set_volume,
-		event_kill_self;
+	/// Event object, world to thread, set to start playback
+	Win32KernelHandle event_start_playback;
 
-	// Thread communicating back
-	Win32KernelHandle
-		thread_running,
-		is_playing,
-		error_happened;
+	/// Event object, world to thread, set to stop playback
+	Win32KernelHandle event_stop_playback;
 
-	const wxChar *error_message;
+	/// Event object, world to thread, set if playback end time was updated
+	Win32KernelHandle event_update_end_time;
+
+	/// Event object, world to thread, set if the volume was changed
+	Win32KernelHandle event_set_volume;
+
+	/// Event object, world to thread, set if the thread should end as soon as possible
+	Win32KernelHandle event_kill_self;
+
+	/// Event object, thread to world, set when the thread has entered its main loop
+	Win32KernelHandle thread_running;
+
+	/// Event object, thread to world, set when playback is ongoing
+	Win32KernelHandle is_playing;
+
+	/// Event object, thread to world, set if an error state has occurred (implies thread is dying)
+	Win32KernelHandle error_happened;
+
+	/// Statically allocated error message text describing reason for error_happened being set
+	const char *error_message;
+
+	/// Playback volume, 1.0 is "unchanged"
 	double volume;
+
+	/// Audio frame to start playback at
 	int64_t start_frame;
+
+	/// Audio frame to end playback at
 	int64_t end_frame;
 
+
+	/// Desired length in milliseconds to write ahead of the playback cursor
 	int wanted_latency;
+
+	/// Multiplier for WantedLatency to get total buffer length
 	int buffer_length;
 
+
+	/// System millisecond timestamp of last playback start, used to calculate playback position
 	DWORD last_playback_restart;
 
+	/// Audio provider to take sample data from
 	AudioProvider *provider;
 
 public:
+	/// @brief Constructor, creates and starts playback thread
+	/// @param provider       Audio provider to take sample data from
+	/// @param WantedLatency Desired length in milliseconds to write ahead of the playback cursor
+	/// @param BufferLength  Multiplier for WantedLatency to get total buffer length
 	DirectSoundPlayer2Thread(AudioProvider *provider, int WantedLatency, int BufferLength);
+	/// @brief Destructor, waits for thread to have died
 	~DirectSoundPlayer2Thread();
 
+	/// @brief Start audio playback
+	/// @param start Audio frame to start playback at
+	/// @param count Number of audio frames to play
 	void Play(int64_t start, int64_t count);
+
+	/// @brief Stop audio playback
 	void Stop();
+
+	/// @brief Change audio playback end point
+	/// @param new_end_frame New last audio frame to play
+	///
+	/// Playback stops instantly if new_end_frame is before the current playback position
 	void SetEndFrame(int64_t new_end_frame);
+
+	/// @brief Change audio playback volume
+	/// @param new_volume New playback amplification factor, 1.0 is "unchanged"
 	void SetVolume(double new_volume);
 
+	/// @brief Tell whether audio playback is active
+	/// @return True if audio is being played back, false if it is not
 	bool IsPlaying();
-	int64_t GetStartFrame();
+
+	/// @brief Get approximate current audio frame being heard by the user
+	/// @return Audio frame index
+	///
+	/// Returns 0 if not playing
 	int64_t GetCurrentFrame();
+
+	/// @brief Get audio playback end point
+	/// @return Audio frame index
 	int64_t GetEndFrame();
-	double GetVolume();
+
+	/// @brief Tell whether playback thread has died
+	/// @return True if thread is no longer running
 	bool IsDead();
 };
 
-
 unsigned int __stdcall DirectSoundPlayer2Thread::ThreadProc(void *parameter)
 {
 	static_cast<DirectSoundPlayer2Thread*>(parameter)->Run();
 	return 0;
 }
 
+/// Macro used to set error_message, error_happened and end the thread
+#define REPORT_ERROR(msg) \
+{ \
+	ResetEvent(is_playing); \
+	error_message = "DirectSoundPlayer2Thread: " msg; \
+	SetEvent(error_happened); \
+	return; \
+}
 
 void DirectSoundPlayer2Thread::Run()
 {
-#define REPORT_ERROR(msg) { error_message = _T("DirectSoundPlayer2Thread: ") _T(msg); SetEvent(error_happened); return; }
-
 	COMInitialization COM_library;
 	try	{ COM_library.Init(); }
 	catch (std::exception e)
@@ -199,7 +282,7 @@ void DirectSoundPlayer2Thread::Run()
 
 	// Create DirectSound object
 	COMObjectRetainer<IDirectSound8> ds;
-	if (FAILED(DirectSoundCreate8(&DSDEVID_DefaultPlayback, &ds.obj, NULL)))
+	if (FAILED(DirectSoundCreate8(&DSDEVID_DefaultPlayback, &ds.obj, nullptr)))
 		REPORT_ERROR("Cound not create DirectSound object")
 
 
@@ -220,7 +303,7 @@ void DirectSoundPlayer2Thread::Run()
 	int aim = waveFormat.nAvgBytesPerSec * (wanted_latency*buffer_length)/1000;
 	int min = DSBSIZE_MIN;
 	int max = DSBSIZE_MAX;
-	DWORD bufSize = MIN(MAX(min,aim),max); // size of entier playback buffer
+	DWORD bufSize = mid(min,aim,max); // size of entire playback buffer
 	DSBUFFERDESC desc;
 	desc.dwSize = sizeof(DSBUFFERDESC);
 	desc.dwFlags = DSBCAPS_GETCURRENTPOSITION2 | DSBCAPS_GLOBALFOCUS;
@@ -241,6 +324,8 @@ void DirectSoundPlayer2Thread::Run()
 	bfr7->Release();
 	bfr7 = 0;
 
+	//wx Log Debug("DirectSoundPlayer2: Created buffer of %d bytes, supposed to be %d milliseconds or %d frames", bufSize, WANTED_LATENCY*BUFFER_LENGTH, bufSize/provider->GetBytesPerSample());
+
 
 	// Now we're ready to roll!
 	SetEvent(thread_running);
@@ -270,7 +355,6 @@ void DirectSoundPlayer2Thread::Run()
 			{
 				// Start or restart playback
 				bfr->Stop();
-				ResetEvent(is_playing);
 
 				next_input_frame = start_frame;
 
@@ -282,21 +366,21 @@ void DirectSoundPlayer2Thread::Run()
 					REPORT_ERROR("Could not reset playback buffer cursor before filling first buffer.")
 
 				HRESULT res = bfr->Lock(buffer_offset, 0, &buf, &buf_size, 0, 0, DSBLOCK_ENTIREBUFFER);
-				while (FAILED(res)) // yes, while, so I can break out of it without a goto!
+				if (FAILED(res))
 				{
 					if (res == DSERR_BUFFERLOST)
 					{
 						// Try to regain the buffer
-						if (SUCCEEDED(bfr->Restore()) &&
-							SUCCEEDED(bfr->Lock(buffer_offset, 0, &buf, &buf_size, 0, 0, DSBLOCK_ENTIREBUFFER)))
+						if (FAILED(bfr->Restore()) ||
+							FAILED(bfr->Lock(buffer_offset, 0, &buf, &buf_size, 0, 0, DSBLOCK_ENTIREBUFFER)))
 						{
-							break;
+							REPORT_ERROR("Lost buffer and could not restore it.")
 						}
-
-						REPORT_ERROR("Lost buffer and could not restore it.")
 					}
-
-					REPORT_ERROR("Could not lock buffer for playback.")
+					else
+					{
+						REPORT_ERROR("Could not lock buffer for playback.")
+					}
 				}
 
 				// Clear the buffer in case we can't fill it completely
@@ -331,57 +415,41 @@ void DirectSoundPlayer2Thread::Run()
 			}
 
 		case WAIT_OBJECT_0+1:
-			{
-				// Stop playing
-				bfr->Stop();
-				ResetEvent(is_playing);
-				playback_should_be_running = false;
-				break;
-			}
+stop_playback:
+			// Stop playing
+			bfr->Stop();
+			ResetEvent(is_playing);
+			playback_should_be_running = false;
+			break;
 
 		case WAIT_OBJECT_0+2:
+			// Set end frame
+			if (end_frame <= next_input_frame)
 			{
-				// Set end frame
-				if (end_frame <= next_input_frame)
-				{
-					bfr->Stop();
-					ResetEvent(is_playing);
-					playback_should_be_running = false;
-				}
-				else
-				{
-					// If the user is dragging the start or end point in the audio display
-					// the set end frame events might come in faster than the timeouts happen
-					// and then new data never get filled into the buffer. See bug #915.
-					goto do_fill_buffer;
-				}
-				break;
+				goto stop_playback;
 			}
 
+			// If the user is dragging the start or end point in the audio display
+			// the set end frame events might come in faster than the timeouts happen
+			// and then new data never get filled into the buffer. See bug #915.
+			goto do_fill_buffer;
+
 		case WAIT_OBJECT_0+3:
-			{
-				// Change volume
-				// We aren't thread safe right now, filling the buffers grabs volume directly
-				// from the field set by the controlling thread, but it shouldn't be a major
-				// problem if race conditions do occur, just some momentary distortion.
-				goto do_fill_buffer;
-			}
+			// Change volume
+			// We aren't thread safe right now, filling the buffers grabs volume directly
+			// from the field set by the controlling thread, but it shouldn't be a major
+			// problem if race conditions do occur, just some momentary distortion.
+			goto do_fill_buffer;
 
 		case WAIT_OBJECT_0+4:
-			{
-				// Perform suicide
-				bfr->Stop();
-				ResetEvent(is_playing);
-				playback_should_be_running = false;
-				running = false;
-				break;
-			}
+			// Perform suicide
+			running = false;
+			goto stop_playback;
 
 		case WAIT_TIMEOUT:
 do_fill_buffer:
 			{
 				// Time to fill more into buffer
-
 				if (!playback_should_be_running)
 					break;
 
@@ -393,10 +461,7 @@ do_fill_buffer:
 				{
 					// Not looping playback...
 					// hopefully we only triggered timeout after being done with the buffer
-					bfr->Stop();
-					ResetEvent(is_playing);
-					playback_should_be_running = false;
-					break;
+					goto stop_playback;
 				}
 
 				DWORD play_cursor;
@@ -429,7 +494,7 @@ do_fill_buffer:
 					    SUCCEEDED(bfr->Lock(0, bufSize, &buf1, &buf1sz, &buf2, &buf2sz, 0)) &&
 					    SUCCEEDED(bfr->Play(0, 0, DSBPLAY_LOOPING)))
 					{
-						wxLogDebug(_T("DirectSoundPlayer2: Lost and restored buffer"));
+						LOG_D("audio/player/dsound") << "Lost and restored buffer";
 						break;
 					}
 					REPORT_ERROR("Lost buffer and could not restore it.")
@@ -478,10 +543,9 @@ do_fill_buffer:
 			break;
 		}
 	}
-
-#undef REPORT_ERROR
 }
 
+#undef REPORT_ERROR
 
 DWORD DirectSoundPlayer2Thread::FillAndUnlockBuffers(void *buf1, DWORD buf1sz, void *buf2, DWORD buf2sz, int64_t &input_frame, IDirectSoundBuffer8 *bfr)
 {
@@ -541,7 +605,6 @@ DWORD DirectSoundPlayer2Thread::FillAndUnlockBuffers(void *buf1, DWORD buf1sz, v
 	return buf1sz + buf2sz;
 }
 
-
 void DirectSoundPlayer2Thread::CheckError()
 {
 	try
@@ -552,10 +615,10 @@ void DirectSoundPlayer2Thread::CheckError()
 			throw error_message;
 
 		case WAIT_ABANDONED:
-			throw _T("The DirectShowPlayer2Thread error signal event was abandoned, somehow. This should not happen.");
+			throw "The DirectShowPlayer2Thread error signal event was abandoned, somehow. This should not happen.";
 
 		case WAIT_FAILED:
-			throw _T("Failed checking state of DirectShowPlayer2Thread error signal event.");
+			throw "Failed checking state of DirectShowPlayer2Thread error signal event.";
 
 		case WAIT_TIMEOUT:
 		default:
@@ -570,8 +633,7 @@ void DirectSoundPlayer2Thread::CheckError()
 	}
 }
 
-
-DirectSoundPlayer2Thread::DirectSoundPlayer2Thread(AudioProvider *provider, int _WantedLatency, int _BufferLength)
+DirectSoundPlayer2Thread::DirectSoundPlayer2Thread(AudioProvider *provider, int WantedLatency, int BufferLength)
 : event_start_playback  (CreateEvent(0, FALSE, FALSE, 0))
 , event_stop_playback   (CreateEvent(0, FALSE, FALSE, 0))
 , event_update_end_time (CreateEvent(0, FALSE, FALSE, 0))
@@ -580,21 +642,19 @@ DirectSoundPlayer2Thread::DirectSoundPlayer2Thread(AudioProvider *provider, int
 , thread_running        (CreateEvent(0,  TRUE, FALSE, 0))
 , is_playing            (CreateEvent(0,  TRUE, FALSE, 0))
 , error_happened        (CreateEvent(0, FALSE, FALSE, 0))
+, wanted_latency(WantedLatency)
+, buffer_length(BufferLength)
+, provider(provider)
 {
 	error_message = 0;
 	volume = 1.0;
 	start_frame = 0;
 	end_frame = 0;
 
-	wanted_latency	= _WantedLatency;
-	buffer_length	= _BufferLength;
-
-	this->provider = provider;
-
-	thread_handle.handle = (HANDLE)_beginthreadex(0, 0, ThreadProc, this, 0, 0);
+	thread_handle = (HANDLE)_beginthreadex(0, 0, ThreadProc, this, 0, 0);
 
 	if (!thread_handle)
-		throw _T("Failed creating playback thread in DirectSoundPlayer2. This is bad.");
+		throw agi::AudioPlayerOpenError("Failed creating playback thread in DirectSoundPlayer2. This is bad.", 0);
 
 	HANDLE running_or_error[] = { thread_running, error_happened };
 	switch (WaitForMultipleObjects(2, running_or_error, FALSE, INFINITE))
@@ -605,21 +665,19 @@ DirectSoundPlayer2Thread::DirectSoundPlayer2Thread(AudioProvider *provider, int
 
 	case WAIT_OBJECT_0 + 1:
 		// error happened, we fail
-		throw error_message;
+		throw agi::AudioPlayerOpenError(error_message, 0);
 
 	default:
-		throw _T("Failed wait for thread start or thread error in DirectSoundPlayer2. This is bad.");
+		throw agi::AudioPlayerOpenError("Failed wait for thread start or thread error in DirectSoundPlayer2. This is bad.", 0);
 	}
 }
 
-
 DirectSoundPlayer2Thread::~DirectSoundPlayer2Thread()
 {
 	SetEvent(event_kill_self);
 	WaitForSingleObject(thread_handle, INFINITE);
 }
 
-
 void DirectSoundPlayer2Thread::Play(int64_t start, int64_t count)
 {
 	CheckError();
@@ -629,8 +687,21 @@ void DirectSoundPlayer2Thread::Play(int64_t start, int64_t count)
 	SetEvent(event_start_playback);
 
 	last_playback_restart = GetTickCount();
-}
 
+	// Block until playback actually begins to avoid race conditions with
+	// checking if playback is in progress
+	HANDLE events_to_wait[] = { is_playing, error_happened };
+	switch (WaitForMultipleObjects(2, events_to_wait, FALSE, INFINITE))
+	{
+	case WAIT_OBJECT_0+0: // Playing
+		LOG_D("audio/player/dsound") << "Playback begun";
+		break;
+	case WAIT_OBJECT_0+1: // Error
+		throw error_message;
+	default:
+		throw agi::InternalError("Unexpected result from WaitForMultipleObjects in DirectSoundPlayer2Thread::Play", 0);
+	}
+}
 
 void DirectSoundPlayer2Thread::Stop()
 {
@@ -639,7 +710,6 @@ void DirectSoundPlayer2Thread::Stop()
 	SetEvent(event_stop_playback);
 }
 
-
 void DirectSoundPlayer2Thread::SetEndFrame(int64_t new_end_frame)
 {
 	CheckError();
@@ -648,7 +718,6 @@ void DirectSoundPlayer2Thread::SetEndFrame(int64_t new_end_frame)
 	SetEvent(event_update_end_time);
 }
 
-
 void DirectSoundPlayer2Thread::SetVolume(double new_volume)
 {
 	CheckError();
@@ -657,7 +726,6 @@ void DirectSoundPlayer2Thread::SetVolume(double new_volume)
 	SetEvent(event_set_volume);
 }
 
-
 bool DirectSoundPlayer2Thread::IsPlaying()
 {
 	CheckError();
@@ -665,10 +733,10 @@ bool DirectSoundPlayer2Thread::IsPlaying()
 	switch (WaitForSingleObject(is_playing, 0))
 	{
 	case WAIT_ABANDONED:
-		throw _T("The DirectShowPlayer2Thread playback state event was abandoned, somehow. This should not happen.");
+		throw "The DirectShowPlayer2Thread playback state event was abandoned, somehow. This should not happen.";
 
 	case WAIT_FAILED:
-		throw _T("Failed checking state of DirectShowPlayer2Thread playback state event.");
+		throw "Failed checking state of DirectShowPlayer2Thread playback state event.";
 
 	case WAIT_OBJECT_0:
 		return true;
@@ -679,27 +747,17 @@ bool DirectSoundPlayer2Thread::IsPlaying()
 	}
 }
 
-
-int64_t DirectSoundPlayer2Thread::GetStartFrame()
-{
-	CheckError();
-
-	return start_frame;
-}
-
-
 int64_t DirectSoundPlayer2Thread::GetCurrentFrame()
 {
 	CheckError();
 
 	if (!IsPlaying()) return 0;
 
-	DWORD milliseconds_elapsed = GetTickCount() - last_playback_restart;
+	int64_t milliseconds_elapsed = GetTickCount() - last_playback_restart;
 
 	return start_frame + milliseconds_elapsed * provider->GetSampleRate() / 1000;
 }
 
-
 int64_t DirectSoundPlayer2Thread::GetEndFrame()
 {
 	CheckError();
@@ -707,15 +765,6 @@ int64_t DirectSoundPlayer2Thread::GetEndFrame()
 	return end_frame;
 }
 
-
-double DirectSoundPlayer2Thread::GetVolume()
-{
-	CheckError();
-
-	return volume;
-}
-
-
 bool DirectSoundPlayer2Thread::IsDead()
 {
 	switch (WaitForSingleObject(thread_running, 0))
@@ -728,130 +777,68 @@ bool DirectSoundPlayer2Thread::IsDead()
 	}
 }
 
-
-
-
-DirectSoundPlayer2::DirectSoundPlayer2()
+DirectSoundPlayer2::DirectSoundPlayer2(AudioProvider *provider)
+: AudioPlayer(provider)
 {
-	thread = 0;
-
 	// The buffer will hold BufferLength times WantedLatency milliseconds of audio
-	WantedLatency = Options.AsInt(_T("Audio dsound buffer latency"));
-	BufferLength = Options.AsInt(_T("Audio dsound buffer length"));
+	WantedLatency = OPT_GET("Player/Audio/DirectSound/Buffer Latency")->GetInt();
+	BufferLength = OPT_GET("Player/Audio/DirectSound/Buffer Length")->GetInt();
 
 	// sanity checking
 	if (WantedLatency <= 0)
 		WantedLatency = 100;
 	if (BufferLength <= 0)
 		BufferLength = 5;
-}
 
+	try
+	{
+		thread = agi::util::make_unique<DirectSoundPlayer2Thread>(provider, WantedLatency, BufferLength);
+	}
+	catch (const char *msg)
+	{
+		LOG_E("audio/player/dsound") << msg;
+		throw agi::AudioPlayerOpenError(msg, 0);
+	}
+}
 
 DirectSoundPlayer2::~DirectSoundPlayer2()
 {
-	CloseStream();
 }
 
-
 bool DirectSoundPlayer2::IsThreadAlive()
 {
-	if (!thread) return false;
-	
-	if (thread->IsDead())
+	if (thread && thread->IsDead())
 	{
-		delete thread;
-		thread = 0;
-		return false;
+		thread.reset();
 	}
 
-	return true;
+	return !!thread;
 }
 
-
-void DirectSoundPlayer2::OpenStream()
-{
-	if (IsThreadAlive()) return;
-
-	try
-	{
-		thread = new DirectSoundPlayer2Thread(GetProvider(), WantedLatency, BufferLength);
-	}
-	catch (const wxChar *msg)
-	{
-		wxLogError(msg);
-		thread = 0;
-	}
-}
-
-
-void DirectSoundPlayer2::CloseStream()
-{
-	if (!IsThreadAlive()) return;
-
-	try
-	{
-		delete thread;
-	}
-	catch (const wxChar *msg)
-	{
-		wxLogError(msg);
-	}
-	thread = 0;
-}
-
-
-void DirectSoundPlayer2::SetProvider(AudioProvider *provider)
-{
-	try
-	{
-		if (IsThreadAlive() && provider != GetProvider())
-		{
-			delete thread;
-			thread = new DirectSoundPlayer2Thread(provider, WantedLatency, BufferLength);
-		}
-
-		AudioPlayer::SetProvider(provider);
-	}
-	catch (const wxChar *msg)
-	{
-		wxLogError(msg);
-	}
-}
-
-
 void DirectSoundPlayer2::Play(int64_t start,int64_t count)
 {
 	try
 	{
-		OpenStream();
 		thread->Play(start, count);
-
-		if (displayTimer && !displayTimer->IsRunning()) displayTimer->Start(15);
 	}
-	catch (const wxChar *msg)
+	catch (const char *msg)
 	{
-		wxLogError(msg);
+		LOG_E("audio/player/dsound") << msg;
 	}
 }
 
-
-void DirectSoundPlayer2::Stop(bool timerToo)
+void DirectSoundPlayer2::Stop()
 {
 	try
 	{
 		if (IsThreadAlive()) thread->Stop();
-
-		if (timerToo && displayTimer) {
-			displayTimer->Stop();
-		}
 	}
-	catch (const wxChar *msg)
+	catch (const char *msg)
 	{
-		wxLogError(msg);
+		LOG_E("audio/player/dsound") << msg;
 	}
 }
 
-
 bool DirectSoundPlayer2::IsPlaying()
 {
 	try
@@ -859,29 +846,13 @@ bool DirectSoundPlayer2::IsPlaying()
 		if (!IsThreadAlive()) return false;
 		return thread->IsPlaying();
 	}
-	catch (const wxChar *msg)
+	catch (const char *msg)
 	{
-		wxLogError(msg);
+		LOG_E("audio/player/dsound") << msg;
 		return false;
 	}
 }
 
-
-int64_t DirectSoundPlayer2::GetStartPosition()
-{
-	try
-	{
-		if (!IsThreadAlive()) return 0;
-		return thread->GetStartFrame();
-	}
-	catch (const wxChar *msg)
-	{
-		wxLogError(msg);
-		return 0;
-	}
-}
-
-
 int64_t DirectSoundPlayer2::GetEndPosition()
 {
 	try
@@ -889,14 +860,13 @@ int64_t DirectSoundPlayer2::GetEndPosition()
 		if (!IsThreadAlive()) return 0;
 		return thread->GetEndFrame();
 	}
-	catch (const wxChar *msg)
+	catch (const char *msg)
 	{
-		wxLogError(msg);
+		LOG_E("audio/player/dsound") << msg;
 		return 0;
 	}
 }
 
-
 int64_t DirectSoundPlayer2::GetCurrentPosition()
 {
 	try
@@ -904,66 +874,35 @@ int64_t DirectSoundPlayer2::GetCurrentPosition()
 		if (!IsThreadAlive()) return 0;
 		return thread->GetCurrentFrame();
 	}
-	catch (const wxChar *msg)
+	catch (const char *msg)
 	{
-		wxLogError(msg);
+		LOG_E("audio/player/dsound") << msg;
 		return 0;
 	}
 }
 
-
 void DirectSoundPlayer2::SetEndPosition(int64_t pos)
 {
 	try
 	{
 		if (IsThreadAlive()) thread->SetEndFrame(pos);
 	}
-	catch (const wxChar *msg)
+	catch (const char *msg)
 	{
-		wxLogError(msg);
+		LOG_E("audio/player/dsound") << msg;
 	}
 }
 
-
-void DirectSoundPlayer2::SetCurrentPosition(int64_t pos)
-{
-	try
-	{
-		if (IsThreadAlive()) thread->Play(pos, thread->GetEndFrame()-pos);
-	}
-	catch (const wxChar *msg)
-	{
-		wxLogError(msg);
-	}
-}
-
-
 void DirectSoundPlayer2::SetVolume(double vol)
 {
 	try
 	{
 		if (IsThreadAlive()) thread->SetVolume(vol);
 	}
-	catch (const wxChar *msg)
+	catch (const char *msg)
 	{
-		wxLogError(msg);
+		LOG_E("audio/player/dsound") << msg;
 	}
 }
 
-
-double DirectSoundPlayer2::GetVolume()
-{
-	try
-	{
-		if (!IsThreadAlive()) return 0;
-		return thread->GetVolume();
-	}
-	catch (const wxChar *msg)
-	{
-		wxLogError(msg);
-		return 0;
-	}
-}
-
-
 #endif // WITH_DIRECTSOUND
