@@ -14,19 +14,21 @@
 //
 // Aegisub Project http://www.aegisub.org/
 
-
-// This implements support for the EBU tech 3264 (1991) subtitling data exchange format.
-// Work on support for this format was sponsored by Bandai.
-
-
-#pragma once
+/// @file subtitle_format_ebu3264.h
+/// @see subtitle_format_ebu3264.cpp
+/// @ingroup subtitle_io
 
 #include "subtitle_format.h"
 
+/// @brief Subtitle writer for the EBU tech 3264 (1991) subtitling data exchange format
+///
+/// Based on specifications obtained at <http://tech.ebu.ch/docs/tech/tech3264.pdf>
+/// Work on support for this format was sponsored by Bandai.
 class Ebu3264SubtitleFormat : public SubtitleFormat {
 public:
-	wxString GetName();
-	wxArrayString GetWriteWildcards();
-	bool CanWriteFile(wxString filename);
-	void WriteFile(wxString filename,wxString encoding);
+	Ebu3264SubtitleFormat();
+	std::vector<std::string> GetWriteWildcards() const override;
+	void WriteFile(const AssFile *src, agi::fs::path const& filename, std::string const& encoding) const override;
+
+	DEFINE_SIMPLE_EXCEPTION(ConversionFailed, agi::InvalidInputException, "subtitle_io/ebu3264/conversion_error")
 };
